/*
www.E2CRE8.be - Brainbox Arduino  - by Bart Huyskens
26/09/2016

Interrupt-based communication bewteen the BBA and a Android smartdevice running a selfmade APPINVENTOR APP
We use the HC06 Bluetooth device (3€ in CHina)
The APP sourcecode and installer are both included in this project and are free to use and alter 

BE AWARE:
That we use a interrupt routine in the microcontroller to be sure that we never miss incoming data.
Arduino IDE is not that great on interrupt handling and that is why we have written some C-code to do the interrupt handling
We use the RX interrupt source of UART1. This generates an interrupt when new data has arrived in UART1 (connected to te HC06 module)

This program
- send 2 bytes - X_Y coordinates from canvas (APP) to uC
- Displays those two bytes on the LCD
- changes the value of led D13 every time a new interrupt occurs
- calculates the power and direction of the two motors (tank style), based on these two coordinates


Connection of HC06 Bluetooth device to Brainbox ARduino:
!!use the 4 pin connector labled 'For HC06' - this fixes all connections for you.
BBA - pin TX    to HC06 pin RXD
BBA - pin RX    to HC06 pin TXD
BBA - +5V       to HC06 pin VCC
BBA - GND       to HC06 pin GND

This program drives an I2C LCD of the type: 
 16 character 2 line I2C Display
 Backpack Interface labelled "YwRobot Arduino LCM1602 IIC V1"  (2€ @ aliexpress)

Connect this LCD as follows:
LCD             Brainbox Arduino
GND             GND
VCC             +5V
SDA             SDA/2
SCL             SCL/3
!! Pull up resistors are optional and not required because ARduino wire lib activates the weak internal pull ups of an AVR - place 4K7 between SDA and 5V and 4K7 between SCL and 5V

To communicate correctly with this I2C LCD you need to install the <LiquidCrystal_I2C.h> library in the arduino IDE
  1- download the "LiquidCrystal_I2C" library as a zip file from https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads
  2- do not unzip the file
  3- in ARduino IDE: Sketch >> include library >> add .ZIP library - select the downloaded zip file
  4- this library is installed under 'mydocs'->Arduino : remove it by deleting it there
*/

# include <avr/io.h>
# include <avr/interrupt.h>
# define F_CPU 16000000         // Xtal op 16MHz
# define USART_BAUDRATE 9600
# define BAUD_PRESCALE (((( F_CPU / 16) + ( USART_BAUDRATE / 2) ) / ( USART_BAUDRATE )) - 1)

#include <Wire.h>  // Comes with Arduino IDE
#include <LiquidCrystal_I2C.h> // Use the extra I2C LCD library: https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads

// set the LCD address to 0x27 for a 20 chars 4 line display
// Set the pins on the I2C chip used for LCD connections:
//                    addr, en,rw,rs,d4,d5,d6,d7,bl,blpol
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address
 
const byte ledPin = 13;
const byte PWM_LEFTMOTOR =  5;    // speed left motor
const byte DIR_LEFTMOTOR =  6;    // direction (forward / backward) left motor
const byte PWM_RIGHTMOTOR =  9;   // speed right motor
const byte DIR_RIGHTMOTOR =  10;  // direction (forward / backward) right motor

volatile byte state = LOW;
volatile byte ReceiveX;
volatile byte ReceiveY;
volatile byte startcomms = 0;

int DC_LEFT = 0;
byte DIR_LEFT = 0;
int DC_RIGHT = 0;
byte DIR_RIGHT = 0;

void setup()
{
  pinMode(ledPin, OUTPUT);
  pinMode(PWM_LEFTMOTOR, OUTPUT);
  pinMode(DIR_LEFTMOTOR, OUTPUT);
  pinMode(PWM_RIGHTMOTOR, OUTPUT);
  pinMode(DIR_RIGHTMOTOR, OUTPUT);

  analogWrite(PWM_LEFTMOTOR, 0);
  digitalWrite(DIR_LEFTMOTOR, 0);
  analogWrite(PWM_RIGHTMOTOR, 0);
  digitalWrite(DIR_RIGHTMOTOR, 0);

  lcd.begin(16,2);// initialize LCD library
  lcd.backlight();              //backlight on
  lcd.setCursor(0,0);           // set cursor to positon x=0, y=0
  lcd.print("TEST BT BBA_04");    // print text on the LCD

  UCSR1B = (1 << RXEN1 ) | (1 << TXEN1 );     // Enable uart1 for transmit and receive
  UCSR1C = (1 << UCSZ10 ) | (1 << UCSZ11 );
  UBRR1H = (BAUD_PRESCALE >> 8);              // Set baud rate register
  UBRR1L = BAUD_PRESCALE;                    // Set baud rate register
  UCSR1B |= (1 << RXCIE1);                   //Enable interrupt when recieve byte is complete
  sei();                                    //Enable global interrupts
}
 
 
void loop()
{
 digitalWrite(ledPin, state);     // flash led D13

if (startcomms == 1){
  if ((ReceiveY >=128)&&(ReceiveX >= 128)) // forward - right
    {
    DIR_LEFT = 0; // forward
    DIR_RIGHT = 0; // forward
    DC_LEFT = ((ReceiveY - 127)*2);
    if (DC_LEFT > 254)DC_LEFT = 255;
    if (DC_LEFT < 0)DC_LEFT = 0;
    DC_RIGHT = ((ReceiveY - 127)*2)-(ReceiveX - 127);
    if (DC_RIGHT > 254)DC_RIGHT = 255;
    if (DC_RIGHT < 0)DC_RIGHT = 0;
    }
    
  else if ((ReceiveY >= 128)&&(ReceiveX <= 127)) // forward left
    {
    DIR_LEFT = 0; // forward
    DIR_RIGHT = 0; // forward
    DC_LEFT = ((ReceiveY - 127)*2)-(127-ReceiveX);
    if (DC_LEFT > 254)DC_LEFT = 255;
    if (DC_LEFT < 0)DC_LEFT = 0;
    DC_RIGHT = ((ReceiveY - 127)*2);
    if (DC_RIGHT > 254)DC_RIGHT = 255;
    if (DC_RIGHT < 0)DC_RIGHT = 0;
    }
    
   else if ((ReceiveY <= 127)&&(ReceiveX <= 127)) // forward left
    {
    DIR_LEFT = 1; // backwards
    DIR_RIGHT = 1; // backwards
    DC_LEFT = 255-(((127-ReceiveY)*2)-(127-ReceiveX));
    if (DC_LEFT > 254)DC_LEFT = 255;
    if (DC_LEFT < 0)DC_LEFT = 0;
    DC_RIGHT = 255-((127-ReceiveY)*2);
    if (DC_RIGHT > 254)DC_RIGHT = 255;
    if (DC_RIGHT < 0)DC_RIGHT = 0;
    }
    else if ((ReceiveY <= 127)&&(ReceiveX >= 128)) // forward left
    {
    DIR_LEFT = 1; // backwards
    DIR_RIGHT = 1; // backwards
    DC_LEFT = 255-((127-ReceiveY)*2);
    if (DC_LEFT > 254)DC_LEFT = 255;
    if (DC_LEFT < 0)DC_LEFT = 0;
    DC_RIGHT = 255-(((127-ReceiveY)*2)-(ReceiveX-127));
    if (DC_RIGHT > 254)DC_RIGHT = 255;
    if (DC_RIGHT < 0)DC_RIGHT = 0;
    }
    else
    {
    DIR_LEFT = 85; // forward
    DIR_RIGHT = 85; // forward
    DC_LEFT =85;
    DC_RIGHT =85;
    }
}
    
 
    analogWrite(PWM_LEFTMOTOR, DC_LEFT);
    digitalWrite(DIR_LEFTMOTOR, DIR_LEFT);
    analogWrite(PWM_RIGHTMOTOR, DC_RIGHT);
    digitalWrite(DIR_RIGHTMOTOR, DIR_RIGHT);

  lcd.clear();                // these LCD lines slow down the whole process a lot. 
  
  lcd.setCursor(0,0);
  lcd.print(ReceiveX); 

  lcd.setCursor(4,0);
  lcd.print(ReceiveY); 

  lcd.setCursor(0,1);
  lcd.print(DIR_LEFT); 

  lcd.setCursor(4,1);
  lcd.print(DC_LEFT); 

  lcd.setCursor(8,1);
  lcd.print(DIR_RIGHT); 

  lcd.setCursor(12,1);
  lcd.print(DC_RIGHT); 
  delay(50);
}

// interrupt routine is called when the first byte is received, We know that the second byte will follow imediately after the first one so we read this second byte in the interrupt routine
ISR (USART1_RX_vect)      // interrupt routine that executes every time a new byte is received
{
    startcomms = 1;
    state = !state;             // inverse the variable "state" to flash led D13
    UCSR1B &= ~(1 << RXCIE1);   // disable RX interrupt
    
    ReceiveX = UDR1;                           // Fetch the received byte value into the variable " ByteReceived " - the RXint flaq is automatically cleared now
    while ((UCSR1A & (1 << RXC1)) == 0) {};         // wait until byte 2 is received
                                                   // Do nothing until data have been received and is ready to be read from UDR 
    ReceiveY = UDR1;                           // receive byte 2
    
    UCSR1B |= (1 << RXCIE1);        //Re-enable RX interrupt
   
}
